package org.pnyka.components.administrator;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.math.BigInteger;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;
import org.bouncycastle.crypto.params.ElGamalParameters;
import org.bouncycastle.crypto.params.ElGamalPublicKeyParameters;
import org.pnyka.components.core.keyholder.KeyHolder;
import org.pnyka.components.core.keyholder.KeyHolderCreator;
import org.pnyka.components.core.tallier.Tallier;
import org.pnyka.cryptolib.crypto.Parameters;
import org.pnyka.database.Database;
import org.pnyka.database.DatabaseQueries;
import org.pnyka.util.PropertiesFiles;
import org.pnyka.util.Validation;


public class BasicAdministrationOperations
{
   //Inserts in database PKI interfaces
   //ea_id is an election authority's id that has been inserted before in database
   public static boolean DefinePKIInterfaces(int ea_id, String ra_url)
    {
        Connection conn = Database.connectDatabase("DB_NAME0", "DB_USERNAME0", "DB_PASSWORD0", "DB_HOST0");
                
        try 
        {      
            Statement stm = conn.createStatement();
            
            ResultSet rs = DatabaseQueries.selectAllFromTable(conn, "config_security_protocol");
            rs.next();
            //if pki security level is defined
            if(rs.getBoolean("pki_locals_voters") == true || rs.getBoolean("pki_central_voters") == true)
            {
                String query = "INSERT INTO pki_interfaces (ea_id, ra_url) VALUES ('" +ea_id+ "', '" +ra_url+ "') WHERE config_el_id = 1";

                int ret = stm.executeUpdate(query);

                if(ret == 1)
                    return true;
                else
                    return false;
            }
            else
            {
                System.out.println("PKI Security level is not defined");
                return false;
            }
        }
        catch (SQLException ex)
        {
            System.out.println("Problem with a transaction");
            ex.printStackTrace();
            return false;
        }        
    }
    
   
    //Parses a txt file, retrieves election catalog and stores data in central database
    public static int CreateCentralElectionCatalog()
    {
        Connection conn = Database.connectDatabase("DB_NAME0", "DB_USERNAME0", "DB_PASSWORD0", "DB_HOST0");
        ResultSet rs2 = null;
        int count_rows = 0;
        
        try 
        {   
            //retrieve path of election catalog which is stored in central db
            ResultSet rs = DatabaseQueries.selectAllFromTable(conn, "config_elections");
            rs2 = DatabaseQueries.selectAllFromTable(conn, "election_catalog");
            count_rows = DatabaseQueries.countRows(rs2);
            
            rs.next();
            String filename = rs.getString("file_el_catalog");
            
            //if file_el_catalog is stored in database
            if(filename == null)
            {
                System.out.println("Election catalog does not exist");
                return 1;// Election catalog does not exist
            }
            else//parse txt file
            {
                BufferedReader br;
                
                br = new BufferedReader(new FileReader(filename));
                
                List lines = new ArrayList();

                for(String line = br.readLine(); line != null; line = br.readLine()) 
                {
                    // split by comma
                    String[] fields = line.split(",");
                    lines.add(fields);
                }

                String[][] election_catalog = (String[][]) lines.toArray(new String[lines.size()][]);

                Statement stm = conn.createStatement();
                String query = "";
                
                conn.setAutoCommit(false);
                
                for(int i = 0; i<election_catalog.length; i++)
                {     
                    if(election_catalog[i][7].equals(" ") == false)
                    {
                        String sqltxt = "SELECT * FROM ea_definition WHERE vote_area ='"+election_catalog[i][7]+"'";
                        ResultSet localea = DatabaseQueries.selectFromTable(conn, sqltxt);
                        if(DatabaseQueries.countRows(localea) != 0)
                        {                        
                            query = "INSERT INTO election_catalog (ident_number, first_name, last_name, father_name, country, city, email_address, voter_area) VALUES" +
                                    "('" +election_catalog[i][0]+ "','" +election_catalog[i][1]+ "','" +election_catalog[i][2]+ "','" +election_catalog[i][3]+ "'," +
                                    "'" +election_catalog[i][4]+ "','" +election_catalog[i][5]+ "','" +election_catalog[i][6]+ "','" +election_catalog[i][7]+ "')";

                            stm.addBatch(query);
                        }
                        else
                        {
                            System.out.println("There is no registration for local EA "+election_catalog[i][7]);
                            return 2; //There is no registration for local EA
                        }
                    }
                    else 
                    {   
                        query = "INSERT INTO election_catalog (ident_number, first_name, last_name, father_name, country, city, email_address) VALUES" +
                                "('" +election_catalog[i][0]+ "','" +election_catalog[i][1]+ "','" +election_catalog[i][2]+ "','" +election_catalog[i][3]+ "'," +
                                "'" +election_catalog[i][4]+ "','" +election_catalog[i][5]+ "','" +election_catalog[i][6]+ "')";

                         stm.addBatch(query);
                    }
                }               
                
                 stm.execute("BEGIN;");
                 int[] rows_affected = stm.executeBatch();
            
                 if(rows_affected.length == election_catalog.length)//if all queries are successful, commit   
                 {
                     stm.execute("COMMIT;");
                     return 0; //election catalog created
                 }
                 else // else rollback
                 {
                    conn.rollback();
                   
                    if(DatabaseQueries.countRows(rs2) != 0)
                        return 4;//Election catalog already defined
                    else
                        return 3; //problem in transaction
                 }
            }
        }
        catch (SQLException ex)
        {
            System.out.println("Problem with a transaction");
            ex.printStackTrace();
            if(count_rows != 0)
                return 4;//Election catalog already defined
            else
                return 3; //problem in transaction
        }       
        catch (Exception ex1)
        {
            return 5; //problem with file
        }       
    }
      
    //creates local eas' election catalogs by retrieving all voters from central election_catalog
    public static int RegistrarActivation()
    {
        Connection conn = Database.connectDatabase("DB_NAME0", "DB_USERNAME0", "DB_PASSWORD0", "DB_HOST0");
        Connection conn_local = Database.connectDatabase("DB_LOCAL_NAME", "DB_LOCAL_USERNAME", "DB_LOCAL_PASSWORD", "DB_LOCAL_HOST");

        try 
        {   
            ResultSet election_catalog = DatabaseQueries.selectAllFromTable(conn_local, "election_catalog");
            int count_voters = DatabaseQueries.countRows(election_catalog);
            if(count_voters != 0)
                return 1;//local election catalog already defined
            
            ResultSet cent_election_catalog = DatabaseQueries.selectAllFromTable(conn, "election_catalog");
            int count_cent_voters = DatabaseQueries.countRows(cent_election_catalog);
            if(count_cent_voters == 0)
                return 2;//central election catalog is not defined
            
            if(count_voters == 0 )
            {
                 PropertiesFiles pf = new PropertiesFiles();
                 Properties properties = pf.ReadProperties("ea.properties");
                 String ea_code = properties.getProperty("EA_CODE");                   
                
                ResultSet ea_voters = DatabaseQueries.selectFromTable(conn, "SELECT * FROM election_catalog WHERE election_catalog.voter_area ='"+ea_code+"'");
                int count_ea_voters = DatabaseQueries.countRows(ea_voters);
                
                Statement stm = conn_local.createStatement();
                String query = "";
                while(ea_voters.next())
                {
                  query = "INSERT INTO election_catalog (ident_number, first_name, last_name, father_name, country, city," +
                          " email_address, voter_area) VALUES('" +ea_voters.getString("ident_number").trim()+ "'," +
                          "'" +ea_voters.getString("first_name").trim()+ "','" +ea_voters.getString("last_name").trim()+ "'," +
                          "'" +ea_voters.getString("father_name").trim()+ "','" +ea_voters.getString("country").trim()+ "'," +
                          "'" +ea_voters.getString("city").trim()+ "','" +ea_voters.getString("email_address").trim()+ "'," +
                          "'" +ea_voters.getString("voter_area").trim()+ "')";   
                  stm.addBatch(query);                  
                }
                
                 stm.execute("BEGIN;");
                 int[] rows_affected = stm.executeBatch();
            
                 if(rows_affected.length == count_ea_voters)//if all queries are successful, commit   
                 {
                     stm.execute("COMMIT;");
                     return 0; //local election catalog created
                 }
                 else // else rollback
                 {
                    conn.rollback();
                    return 3; //problem in transaction
                 }  
             }   
            return 0;
        }
        catch (SQLException ex)
        {
            return 3; //problem in transaction
        } 
    }
    //Stops election process by renaming web applications' files of each local ea 
    //returns false if voting is in process
    //The function works only when is executed locally, in the same pc that local ea runs  
    public static boolean StopElectionProcess()
    {        
        try
        {
            //Check if election process is in process
            int valid_date_time = Validation.validateDateAndTime("tallier");
            
            if(valid_date_time == 2)
            {
                Connection conn = Database.connectDatabase("DB_NAME0", "DB_USERNAME0", "DB_PASSWORD0", "DB_HOST0");
                PropertiesFiles pf = new PropertiesFiles();
                Properties properties = pf.ReadProperties("system.properties");

                ResultSet local_eas = DatabaseQueries.selectFromTable(conn, "SELECT * FROM ea_definition WHERE is_local = TRUE");
                int count_local_eas = DatabaseQueries.countRows(local_eas);

                if(count_local_eas > 0)//if there are local eas, move local eas' web app folders
                {
                    String[] web_apps_paths = new String[count_local_eas];

                    for(int i=0; i<web_apps_paths.length; i++)
                        web_apps_paths[i] = properties.getProperty("LOCAL_EA"+(i+1));

                    for(int i=0; i<web_apps_paths.length; i++)
                    {
                        // File (or directory) with old name
                        File file1 = new File(web_apps_paths[i]);

                        // File (or directory) with new name
                        File file2 = new File( web_apps_paths[i]+"removed");

                        // Rename file (or directory)
                        boolean success = file1.renameTo(file2);
                        if (!success)          
                        {
                            System.out.println("File was not successfully renamed");  
                            return false;
                        }
                        else
                            return true;
                    } 
                    return true;
                }
                else if(count_local_eas == 0)//if there are not local eas, move central ea's web application folder
                {
                    String web_app_path = properties.getProperty("LOCAL_EA0");
                    
                    // File (or directory) with old name
                    File file1 = new File(web_app_path);

                        // File (or directory) with new name
                        File file2 = new File( web_app_path+"removed");

                        // Rename file (or directory)
                        boolean success = file1.renameTo(file2);
                        if (!success)          
                        {
                            System.out.println("File was not successfully renamed");  
                            return false;
                        }
                        else
                            return true;
                }
                else
                    return true;
            }
            else
                return false;            
            
        }
        catch (SQLException ex)
        {
            System.out.println("Problem with a transaction");
            ex.printStackTrace();
            return false;
        }
    }
   
    //Drops the urls of the election authorities that are contained in 
    //The function works only when is executed locally, in the same pc that local ea runs
    public static boolean DropElectionProcess()
    {        
        try
        {
            Connection conn = Database.connectDatabase("DB_NAME0", "DB_USERNAME0", "DB_PASSWORD0", "DB_HOST0");
            PropertiesFiles pf = new PropertiesFiles();
            Properties properties = pf.ReadProperties("system.properties");

            ResultSet local_eas = DatabaseQueries.selectFromTable(conn, "SELECT * FROM ea_definition WHERE is_local = TRUE");
            int count_local_eas = DatabaseQueries.countRows(local_eas);

            if(count_local_eas > 0)//if there are local eas, move local eas' web app folders
            {
                String[] web_apps_paths = new String[count_local_eas];

                for(int i=0; i<web_apps_paths.length; i++)
                    web_apps_paths[i] = properties.getProperty("LOCAL_EA"+(i+1));

                for(int i=0; i<web_apps_paths.length; i++)
                {
                    
                        // File (or directory) with old name
                        File file1 = new File(web_apps_paths[i]);

                        // File (or directory) with new name
                        File file2 = new File( web_apps_paths[i]+"removed");

                        // Rename file (or directory)
                        boolean success = file1.renameTo(file2);
                        if (!success)          
                        {
                            System.out.println("File was not successfully renamed");  
                            return false;
                        }
                        else
                            return true;
                    } 
                    return true;
                }
                else if(count_local_eas == 0)//if there are not local eas, move central ea's web app folder
                {
                    String web_app_path = properties.getProperty("LOCAL_EA0");
                    
                    // File (or directory) with old name
                    File file1 = new File(web_app_path);

                        // File (or directory) with new name
                        File file2 = new File( web_app_path+"removed");

                        // Rename file (or directory)
                        boolean success = file1.renameTo(file2);
                        if (!success)          
                        {
                            System.out.println("File was not successfully renamed");  
                            return false;
                        }
                        else
                            return true;
                }
                else
                    return true;
        }
        catch (SQLException ex)
        {
            System.out.println("Problem with a transaction");
            ex.printStackTrace();
            return false;
        }
    }
    
    public static int startTallier()
    {
         ResultSet tallier_rs = null;
         Connection conn = Database.connectDatabase("DB_LOCAL_NAME", "DB_LOCAL_USERNAME", "DB_LOCAL_PASSWORD", "DB_LOCAL_HOST");
                 
         int valid_date_time = -1;
         try
         {
            valid_date_time = Validation.validateDateAndTime("tallier");
         }
         catch (NullPointerException ex)
         {
             return 4;
         }
         
         int tallier_rows = 0;    
         
         if(valid_date_time == 2)
         {
             try
             {  
                 tallier_rs = DatabaseQueries.selectAllFromTable(conn, "tallier");
                 tallier_rows = DatabaseQueries.countRows(tallier_rs);
             }
             catch(SQLException e)
             {
                 return 1;// problem with transaction
             }
                              
             if(tallier_rows == 0)
             {
                Tallier tallier = new Tallier();
                tallier.homomorphicAdd();                
                
                tallier.publishEncryptedResults();               
                
                BigInteger[][] decrypted = KeyHolder.sendDecryptedResultToTallier();
              
                tallier.computeResults(decrypted);                
               
              return 0; //final results are stored in central database
            }
            else
                return 2; //final results are already computed
        }
        else
            return 3; //voting in process
    
    }
        
    //activates keyholders 
    public static int activateKeyholders()
    {        
        ResultSet  keyholders = null;
        Connection conn = null;
        ResultSet keyholders_keys = null;
        int count = 0;
        KeyHolder[] keyholderArray = null;
        ElGamalParameters params = null;
        int key_length = 0;   
        
        try
        {
            conn = Database.connectDatabase("DB_NAME0", "DB_USERNAME0", "DB_PASSWORD0", "DB_HOST0");
            keyholders_keys = DatabaseQueries.selectAllFromTable(conn, "keyholders_keys");
            count = DatabaseQueries.countRows(keyholders_keys);
            ResultSet config_security_protocol = DatabaseQueries.selectAllFromTable(conn, "config_security_protocol");
            config_security_protocol.next();
            key_length = config_security_protocol.getInt("key_length");//retrieve key length
        }
        catch(SQLException e)
        {
            return 3;//problem with transaction
        }
        
        if(count == 0)
        {
            //ARXIKOPOIHSEIS - paragwgh parametrwn p,g
            Parameters prm = new Parameters(key_length);///mikos kleidioy
            params = prm.generateElGamalParameters();//telos paragwghs p,g        

            KeyHolderCreator khCreator = new KeyHolderCreator();
            ElGamalPublicKeyParameters h = new ElGamalPublicKeyParameters(BigInteger.ONE, params);//arxiko public
     
            try
            {
                keyholders = DatabaseQueries.selectAllFromTable(conn, "keyholders");
                keyholderArray = new KeyHolder[DatabaseQueries.countRows(keyholders)];

                while (keyholders.next()) 
                {
                    //step 1 - keys generation
                    keyholderArray[keyholders.getRow()-1] = khCreator.createKeyHolder(keyholders.getInt("kh_id"), params);
                    h = keyholderArray[keyholders.getRow()-1].computePartPublicKey(h);//h: to teliko public key
                }     

                //store  p, g  and public key in db
                Statement stm = conn.createStatement();
                String sqlText = "INSERT INTO parameters (parameter_p, parameter_g, public_key) VALUES ('" + params.getP()+ "', '"+ params.getG()+"','"+h.getY()+"')";
                stm.execute(sqlText);
                
                return 1;//keyholders activation successful

            }
            catch(SQLException e)
            {
                return 3;//problem with transaction
            }
        }
        else 
        {
            return 2;//keyholders are already activated
        }        
    }  
    

}
