package org.pnyka.components.core.tallier;

import java.math.BigDecimal;
import java.math.BigInteger;
import java.sql.Connection;
import java.sql.Date;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import org.bouncycastle.crypto.params.ElGamalParameters;
import org.pnyka.components.core.votemanager.ProtocolParameters;
import org.pnyka.cryptolib.crypto.CipherText;
import org.pnyka.cryptolib.math.DiscreteLog;
import org.pnyka.database.Database;
import org.pnyka.database.DatabaseQueries;

public class Tallier
{
    Connection conn = null, conn_central = null;
    Date start = null, stop = null;
    boolean timestamp = false;
    ElGamalParameters params = null;
    CipherText[] encrResult = null;
    boolean exist_pki = false;
    
    //Tallier constructor 
    public Tallier()
    {
        ProtocolParameters protoParams = null;
        
        try
        {
             ResultSet config_sec_protocol = null, config_elections = null;
             boolean zkp1 = false, zkp2 = false, signatures = false, timestamp = false, reencryption = false;
             conn = Database.connectDatabase("DB_LOCAL_NAME", "DB_LOCAL_USERNAME", "DB_LOCAL_PASSWORD", "DB_LOCAL_HOST");
             conn_central = Database.connectDatabase("DB_NAME0", "DB_USERNAME0", "DB_PASSWORD0", "DB_HOST0");
             config_sec_protocol = DatabaseQueries.selectAllFromTable(conn_central, "config_security_protocol");
             config_sec_protocol.next();
             zkp1 = config_sec_protocol.getBoolean("zkp_protocol");
             signatures = config_sec_protocol.getBoolean("signatures_protocol");
             timestamp = config_sec_protocol.getBoolean("timestamps_protocol");
             protoParams = new ProtocolParameters(reencryption, zkp1, zkp2, timestamp, signatures);
             
             config_elections = DatabaseQueries.selectAllFromTable(conn_central, "config_elections");
             config_elections.next();
             start = config_elections.getDate("starting_date");
             stop = config_elections.getDate("stopping_date");

             ResultSet parameters = DatabaseQueries.selectAllFromTable(conn_central, "parameters");
             parameters.next();
             BigInteger p = parameters.getBigDecimal("parameter_p").toBigInteger();
             BigInteger g = parameters.getBigDecimal("parameter_g").toBigInteger();
             
             params = new ElGamalParameters(p, g);   
             
             exist_pki = config_sec_protocol.getBoolean("pki_locals_voters");
        }
        catch(SQLException ex){}
        
        this.timestamp = protoParams.timestamp;
    }
    
    //homomorphic addition of votes
    public void homomorphicAdd()
    {
        ResultSet allVotes = null;//or most recent 
      
        if(timestamp == true && exist_pki == true)//if the voter can submit multiple votes, with only the last be valid
            allVotes = getMostRecentVotes("encrypted_votes");            
        else if(timestamp == true && exist_pki == false) //if the voter can submit multiple votes and all votes are valid
            allVotes = getVotes("encrypted_votes");
        else
            allVotes = getVotes("encrypted_votes");            
        
        ResultSet params = null;
        ElGamalParameters prm = null;
        
        try
        {
            params = DatabaseQueries.selectAllFromTable(conn_central, "parameters");

            params.next();
            prm = new ElGamalParameters(((BigDecimal)params.getObject("parameter_p")).toBigInteger(), ((BigDecimal)params.getObject("parameter_g")).toBigInteger());

            int cand_number = DatabaseQueries.countRows(DatabaseQueries.selectFromTable(conn,"SELECT * FROM config_candidates ORDER BY candidate_m"));
            encrResult = new CipherText[cand_number];
            
            for (int i=0; i<encrResult.length; i++)//initialization 
                encrResult[i] = new CipherText(BigInteger.ONE, BigInteger.ONE);//Z*...*Z|C*...*C||.....||Z*...*Z|C*...*C
                
            int allVotes_count = DatabaseQueries.countRows(allVotes);
            BigInteger[] z = new BigInteger[allVotes_count];
            BigInteger[] c = new BigInteger[allVotes_count];
            
            int array_index = 0;
            
            while(allVotes.next())
            {
               z[array_index] = allVotes.getBigDecimal("int_z").toBigInteger();
               c[array_index] = allVotes.getBigDecimal("int_c").toBigInteger();
               
               array_index++;
            }
            
            int ptr = 0;
            
            while(ptr<z.length)
            {
               int ind = 0;
                                
                if((ptr+1)% encrResult.length  == 0)
                   ind = encrResult.length -1;
                else
                   ind = (ptr+1)% encrResult.length  - 1;                 
                    
                encrResult[ind] =  new CipherText(encrResult[ind].getZ().multiply(z[ptr]).mod(prm.getP()), encrResult[ind].getC().multiply(c[ptr]).mod(prm.getP()));  
                   
                ptr++;
            }
        }
        catch (SQLException e){}        
    }
    

    //retrieves most recent votes from database
    private ResultSet getMostRecentVotes(String votesTable)
    {
       ResultSet votes = null;
       Statement sql = null;       
       String sqlText; 
                       
        try
        {  
            sql = conn.createStatement(ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_UPDATABLE);
            //sqlText = "SELECT a.vote_id, a.voter_id, a.ident_num, a.int_z, a.int_c, a.timestamp FROM (SELECT ident_num,  max(timestamp) FROM encrypted_votes GROUP BY voter_id) AS b INNER JOIN encrypted_votes AS a ON b.ident_num = a.ident_num AND max = a.timestamp";
            sqlText = "SELECT a.vote_id, a.voter_id, a.int_z, a.int_c, a.timestamp FROM (SELECT voter_id,  max(timestamp) FROM encrypted_votes GROUP BY voter_id) AS b INNER JOIN encrypted_votes AS a ON b.voter_id = a.voter_id AND max = a.timestamp";
            votes =  DatabaseQueries.selectFromTable(conn, sqlText);//sql.executeQuery(sqlText);  
            
                   
            return votes;
        }
        catch (SQLException e){}
                
        return votes;
    }
    
    //retrieves all recent votes from database
    private ResultSet getVotes(String votesTable)
    {
        ResultSet votes = null;
        
        try
        {
            votes = DatabaseQueries.selectAllFromTable(conn, votesTable);
            return votes;
        }
        catch (SQLException e){}
                
        return null;
    }
     
    //inserts encrypted results in database
    public void publishEncryptedResults() 
    {       
        try
        {
             ResultSet config_candidates = DatabaseQueries.selectFromTable(conn, "SELECT * FROM config_candidates ORDER BY candidate_id");

             Statement stm = conn.createStatement();

             for(int i=0; i<encrResult.length; i++)
             {    
                config_candidates.next();
                int ballot_id = config_candidates.getInt("ballot_id");
                int candidate_id = config_candidates.getInt("candidate_id");

                String sqlText = "INSERT INTO tallier (ballot_id, candidate_id, enc_res_z, enc_res_c) VALUES  ('" +ballot_id + "','" +candidate_id+ "','"+encrResult[i].getZ()+"','" +encrResult[i].getC()+ "')";
                stm.execute(sqlText); 
             }
        }
        catch (SQLException ex)
        {
            System.out.println("Problem with a transaction");
            ex.printStackTrace();
        }
    }
    
    //inserts encrypted results in database
    public void SendEncryptedResultToKeyholders()
    {
        publishEncryptedResults();
    }
    
    //computes final results
    public void computeResults(BigInteger[][] decrypted)
    {
        for(int i=0; i<decrypted.length; i++)
        {           
            int candidate_m = decrypted[i][0].intValue();
            BigInteger plain = decrypted[i][1];
            
            BigInteger result = DiscreteLog.bruteForce(BigInteger.valueOf(candidate_m), plain, params.getP());
            
            int r = result.mod(params.getP().subtract(BigInteger.ONE)).intValue();
            publishResults(candidate_m, r);
        }
    }
    
    //inserts final results in database
    public void publishResults(int m, int r)
    {
         try
         {
             Statement stm = conn.createStatement();
             ResultSet config_candidates = DatabaseQueries.selectFromTable(conn, "SELECT * FROM config_candidates WHERE candidate_m ='"+m+"' ");
             config_candidates.next();

             int candidate_id = config_candidates.getInt("candidate_id");

             String sqlText = "UPDATE tallier SET candidate_votes = '"+ r+ "' WHERE candidate_id='"+candidate_id+"'";
             stm.execute(sqlText); 
         }
         catch (SQLException ex)
         {
             System.out.println("Problem with a transaction");
             ex.printStackTrace();
         }
    }
    
    private void checkTimeStamp()
    {
        
    }  
    
}