package org.pnyka.components.registrar;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Properties;
import org.pnyka.database.Database;
import org.pnyka.database.DatabaseQueries;
import org.pnyka.util.PropertiesFiles;


public class Registration
{   
    //Creates credentials for election catalog's voters, if no local ea's exist
    public static int StartCentralRegistrar()
    {
        Connection conn = Database.connectDatabase("DB_NAME0", "DB_USERNAME0", "DB_PASSWORD0", "DB_HOST0");
        
        try 
        {            
            //look for election catalog file path 
            ResultSet config_elections = DatabaseQueries.selectAllFromTable(conn, "config_elections");
            String file_el_catalog = null; 
            
            if(DatabaseQueries.countRows(config_elections) != 0)
            {
                config_elections.next();
                file_el_catalog = config_elections.getString("file_el_catalog");
            }
            else
                return 1;// file_el_catalog is not defined
            
            if(file_el_catalog.equals(" ") || file_el_catalog == null) //if election catalog's path is not inserted, return 1
                return 1;// file_el_catalog is not defined
            else //else retrieve the contents of election_catalog table and create usernames and passwords
            {
                ResultSet voter_credentials = DatabaseQueries.selectAllFromTable(conn, "voter_credentials");
                int count_voter_credentials = DatabaseQueries.countRows(voter_credentials);
                
                if(count_voter_credentials != 0)
                    return 7;
                
                int count_local_eas = DatabaseQueries.countRows(DatabaseQueries.selectFromTable(conn, "SELECT * FROM ea_definition WHERE is_local = TRUE"));
                ResultSet election_catalog = DatabaseQueries.selectAllFromTable(conn, "election_catalog");//activate central registrar

                if(DatabaseQueries.countRows(election_catalog) == 0)
                    return 6;//registrar must be first activated

                int count_voters = DatabaseQueries.countRows(election_catalog);
                Statement stm = conn.createStatement();

                String[] usrs = CreateArrayOfUsernames(count_voters);//array usrs contains all usernames
                String[] psws = CreateArrayOfPasswords(count_voters, 10);//array psws contains all passwords                   

                int voter_id, ptr = 0;
                String ident_num;

                conn.setAutoCommit(false);

                while(election_catalog.next())
                {
                    voter_id = election_catalog.getInt("voter_id");
                    ident_num = election_catalog.getString("ident_number");

                    stm.addBatch("INSERT INTO voter_credentials (voter_id, ident_num, username, password) VALUES('" +voter_id+ "','" +ident_num+ "','" +usrs[ptr]+ "','" +psws[ptr]+ "')");
                    ptr++;                                               
                }

                //execute transaction
                stm.execute("BEGIN;");
                int[] rows_affected = stm.executeBatch();

                if(rows_affected.length == count_voters)//if all queries are successful, commit   
                {
                    stm.execute("COMMIT;");
                    if(count_local_eas == 0)
                        return 0;// ok
                    else
                        return 3;//local EA's exist and local registrar must be started 
                }
                else // else rollback
                {
                    conn.rollback();
                    return 2;//broblem with transaction
                }
             }
                            
        }
        catch (SQLException ex)
        {
            System.out.println("Problem with a transaction");
            ex.printStackTrace();
            return 2;//broblem with transaction
        }
        catch (NumberFormatException ex2)
        {
            System.out.println("Cannot convert string to integer");
            ex2.printStackTrace();
            return 5;//number format exception
        }
        
    }
    
    //Creates credentials for election catalog's voters, if local ea's exist
    public static int StartLocalRegistrar()
    {
        Connection conn = Database.connectDatabase("DB_NAME0", "DB_USERNAME0", "DB_PASSWORD0", "DB_HOST0");
        Connection conn_local = Database.connectDatabase("DB_LOCAL_NAME", "DB_LOCAL_USERNAME", "DB_LOCAL_PASSWORD", "DB_LOCAL_HOST");
        
        try 
        {            
            //look for election catalog file path 
            ResultSet election_catalog = DatabaseQueries.selectAllFromTable(conn, "election_catalog");
            ResultSet voter_credentials = DatabaseQueries.selectAllFromTable(conn, "voter_credentials");
            
            if(DatabaseQueries.countRows(election_catalog) == 0)
                return 1;//central election_catalog is not defined
            else if(DatabaseQueries.countRows(voter_credentials) == 0)
                return 2; //central voter_credentials is not defined
            
           ResultSet local_voters = DatabaseQueries.selectAllFromTable(conn_local, "voter_credentials");
           if(DatabaseQueries.countRows(local_voters) != 0)
               return 4;
            
           PropertiesFiles pf = new PropertiesFiles();
           Properties properties = pf.ReadProperties("ea.properties");
           String ea_code = properties.getProperty("EA_CODE");
            
           Statement stm = conn.createStatement(ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_UPDATABLE);
           ResultSet ea_voters = stm.executeQuery("SELECT voter_credentials.voter_id, ident_num, username, password  FROM election_catalog, voter_credentials WHERE election_catalog.voter_area = '"+ea_code+"' AND election_catalog.voter_id = voter_credentials.voter_id");
           int count_ea_voters = DatabaseQueries.countRows(ea_voters);

           Statement stm1 = conn_local.createStatement();
           String query = "";
           while(ea_voters.next())
           {
              query = "INSERT INTO voter_credentials (voter_id, ident_num, username, password) " +
                      "VALUES('" +ea_voters.getString("voter_id").trim()+ "','" +ea_voters.getString("ident_num").trim()+ "'," +
                      "'" +ea_voters.getString("username").trim()+ "','" +ea_voters.getString("password").trim()+ "')";   
              System.out.println(query);
              stm1.addBatch(query);                  
           }

             stm1.execute("BEGIN;");
             int[] rows_affected = stm1.executeBatch();

             if(rows_affected.length == count_ea_voters)//if all queries are successful, commit   
             {
                 stm1.execute("COMMIT;");
                 return 0; //local election catalog created
             }
             else // else rollback
             {
                conn_local.rollback();
                return 3; //problem in transaction
             }                              
        }
        catch (SQLException ex1)
        {
            System.out.println("Problem with a transaction");
            ex1.printStackTrace();
            return 3;//broblem with transaction
        }              
    }    
    
    //Returns an array with 'numberOfPasswords' different passwords
    //length: how many characters the password has
    public static String[] CreateArrayOfPasswords(int numberOfPasswords, int length) 
    {      
        PasswordGenerator pswGen = new PasswordGenerator();
               
        String[] psw = new String[numberOfPasswords];
        for(int i=0; i<psw.length; i++)
        {
            psw[i] = "";
        }
        
        for(int i=0; i<psw.length; i++)
        {
            String s = pswGen.createPSW(length);
                                    
            while(pswGen.exists(s, psw) == true)
            {
              s = pswGen.createPSW(length);
            }            
            psw[i] = s;  
        }         
        return psw;
     }
    
    public static String[] CreateArrayOfUsernames(int numberOfUsernames) 
    {    
        String[] usr = new String[numberOfUsernames];
        
        UsernameGenerator unGen = new UsernameGenerator();
        
        for(int i=0; i<numberOfUsernames; i++)
        {
            usr[i] = unGen.generate(8);
        }        
        return usr;
    }
    
}
