package org.pnyka.cryptolib.ZKP;

import java.math.BigInteger;
import java.security.SecureRandom;
import org.bouncycastle.crypto.params.ElGamalPublicKeyParameters;

/**
 * Class for common attributes and methods of all ZKP.
 */
public class BaseZKP 
{
    public BigInteger p, g, h;
    
   /**
    * @param p parameter p
    * @param g parameter g
    * @param h parameter h
    */   
    public BaseZKP(BigInteger p, BigInteger g, BigInteger h)
    {
        this.p = p;
        this.g = g;
        this.h = h;        
    }
        
   /**
    * Selects a random number mod G
    * @param p prime numper p
    * @return r random number
    */   
    public static BigInteger selectRandom(BigInteger p)
    {
        SecureRandom sr = new SecureRandom();   
        int pBitLength = p.bitLength();
        BigInteger r = new BigInteger(pBitLength, sr);

        while (r.equals(BigInteger.ZERO) || (r.compareTo(p.subtract(BigInteger.valueOf(1))) > 0))
        {
            r = new BigInteger(pBitLength, sr);
        }             
        
        return r.mod(p);
    }
    
}
