package org.pnyka.components.administrator;

import java.io.IOException;
import java.sql.Connection;
import java.sql.Date;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Time;
import javax.servlet.jsp.JspWriter;
import org.pnyka.database.Database;
import org.pnyka.database.DatabaseQueries;

//The class contains all functions needed for system parameterization
public class SystemParameterization 
{  
    //Stores starting and stopping date and time in database
    public static boolean DefineStartStopElection(Date starting_date, Date stopping_date, Time starting_time, Time stopping_time)
    {
        Connection conn = Database.connectDatabase("DB_NAME0", "DB_USERNAME0", "DB_PASSWORD0", "DB_HOST0");
        
        try 
        {            
            Statement stm = conn.createStatement();
            
            String query = null;
            
            int conf_el_count = DatabaseQueries.countRows(DatabaseQueries.selectAllFromTable(conn, "config_elections"));

            if(conf_el_count == 1)//if other parameters are inserted in 'config_elections' table before 'starting_date', etc.
                query = "UPDATE config_elections SET starting_date = '"+starting_date+"', stopping_date = '"+stopping_date+"', starting_time = '"+starting_time+"', stopping_time = '"+stopping_time+"'  WHERE config_el_id = 1";
            else if(conf_el_count == 0)
                query = "INSERT INTO config_elections (starting_date, stopping_date, starting_time, stopping_time) VALUES ('"+starting_date+ "', '"+stopping_date+ "', '"+starting_time+ "', '"+stopping_time+ "')";

            int ret = stm.executeUpdate(query);
            
            if(ret == 1)
                return true;
            else
                return false;
        }
        catch (SQLException ex)
        {
            System.out.println("Problem with a transaction");
            ex.printStackTrace();
            return false;
        }        
    }
    
    public static boolean DefineElectionProcess(String ballot_description, int candidate_num, boolean is_white,  boolean is_invalid, String vote_area, String[] candidate_description, int[] candidate_m)
    {
        boolean stored = true;
        
        Connection conn = Database.connectDatabase("DB_NAME0", "DB_USERNAME0", "DB_PASSWORD0", "DB_HOST0");
        int ballot_id = -1;
        
        try 
        {  
            //find ballot_id (seq) of the next ballot (the ballot that will be insertes by this function)
            Statement stm = conn.createStatement();
            ResultSet seq = stm.executeQuery("SELECT max(ballot_id) FROM config_ballots");
            seq.next();
            ballot_id = seq.getInt(1)+1;             
            
            conn.setAutoCommit(false);
            if(vote_area == null)
                stm.addBatch("INSERT INTO config_ballots (ballot_id, ballot_description, candidate_num, is_white, is_invalid) VALUES ('" +ballot_id+ "','" +ballot_description+ "','" +candidate_num+ "','" +is_white+ "','" +is_invalid+ "')");
            else
                stm.addBatch("INSERT INTO config_ballots (ballot_id, ballot_description, candidate_num, is_white, is_invalid, vote_area) VALUES ('" +ballot_id+ "','" +ballot_description+ "','" +candidate_num+ "','" +is_white+ "','" +is_invalid+ "', '" +vote_area+ "')");
            
            for(int i=0; i< candidate_num; i++)
                stm.addBatch("INSERT INTO config_candidates (ballot_id, candidate_description, candidate_m) VALUES ('" +ballot_id+ "','" +candidate_description[i]+ "','" +candidate_m[i]+ "')");
                            
            stm.execute("BEGIN;");
            int[] rows_affected = stm.executeBatch();
            
            if(rows_affected.length == candidate_num+1)//if all queries are successful, commit               
                stm.execute("COMMIT;");
            else // else rollback
            {
                conn.rollback();                             
                stored = false;
            }           
        }
        catch (SQLException ex)
        {
            System.out.println("Problem with a transaction");
            ex.printStackTrace();
            stored = false;
        }
        
        return stored;     
    }
    
    public static boolean DefineElectionParticipation(String file_el_catalog)
    {
        Connection conn = Database.connectDatabase("DB_NAME0", "DB_USERNAME0", "DB_PASSWORD0", "DB_HOST0");
        
        try 
        {            
            Statement stm = conn.createStatement();
            
            String query = null;
            
            int conf_el_count = DatabaseQueries.countRows(DatabaseQueries.selectAllFromTable(conn, "config_elections"));
            
            if (conf_el_count == 1) //if other parameters are inserted in 'config_elections' table before 'file_el_catalog'
                query = "UPDATE config_elections SET file_el_catalog = '"+file_el_catalog+"' WHERE config_el_id = 1";
            else if(conf_el_count == 0)
                query = "INSERT INTO config_elections (file_el_catalog) VALUES ('" +file_el_catalog+ "')";
            
            System.out.println(query);
            int ret = stm.executeUpdate(query);
            
            if(ret == 1)
                return true;
            else
                return false;
        }
        catch (SQLException ex)
        {
            System.out.println("Problem with a transaction");
            ex.printStackTrace();
            return false;
        }        
    }
    
    public static boolean DefineValidVotes(boolean one_vote, boolean many_votes)
    {
        Connection conn = Database.connectDatabase("DB_NAME0", "DB_USERNAME0", "DB_PASSWORD0", "DB_HOST0");
        
        try 
        {   
            Statement stm = conn.createStatement();
            
            String query = null;
            
            int conf_el_count = DatabaseQueries.countRows(DatabaseQueries.selectAllFromTable(conn, "config_elections"));
            
            if (conf_el_count == 1)//if other parameters are inserted in 'config_elections' table before 'one_vote' and 'many_votes''
                query = "UPDATE config_elections SET one_vote = '"+one_vote+"', many_votes = '"+many_votes+"' WHERE config_el_id = 1";
            else if(conf_el_count == 0)
                query = "INSERT INTO config_elections (one_vote, many_votes) VALUES ('" +one_vote+ "','" +many_votes+ "')";
                       
            int ret = stm.executeUpdate(query);
            
            if(ret == 1)
                return true;
            else
                return false;
        }
        catch (SQLException ex)
        {
            System.out.println("Problem with a transaction");
            ex.printStackTrace();
            return false;
        }        
    }
    
    public static boolean DefineProtocolParameters(boolean zkp_protocol, boolean signatures_protocol, boolean timestamps_protocol, int key_length, int threshold) 
    {
        Connection conn = Database.connectDatabase("DB_NAME0", "DB_USERNAME0", "DB_PASSWORD0", "DB_HOST0");
                 
        try 
        {
            Statement stm = conn.createStatement();

            String query = null;

            int conf_sec_proto_count = DatabaseQueries.countRows(DatabaseQueries.selectAllFromTable(conn, "config_security_protocol"));

            if (conf_sec_proto_count == 1)//if other parameters are inserted in 'config_security_protocol' table before 'zkp_protocol', etc.
                query = "UPDATE config_security_protocol SET zkp_protocol = '"+zkp_protocol+"' , signatures_protocol = '"+signatures_protocol+"', timestamps_protocol = '"+timestamps_protocol+"', key_length = "+key_length+", threshold = "+threshold+" WHERE config_sec_pr_id = 1";
            else if(conf_sec_proto_count == 0)
                query = "INSERT INTO config_security_protocol (zkp_protocol, signatures_protocol, timestamps_protocol, key_length, threshold) VALUES ('" +zkp_protocol+ "', '" +signatures_protocol+ "','" +timestamps_protocol+ "', '" +key_length+ "','" +threshold+ "')";

            System.out.println(query);
            
            int ret = stm.executeUpdate(query);

            if(ret == 1)
                return true;
            else
                return false;
        }
        catch (SQLException ex)
        {
            System.out.println("Problem with a transaction");
            return false;
        } 
    }
    
    public static boolean DefineSecurityLevel(boolean pki_central_voters, boolean pki_locals_voters, boolean vpn_central_locals)
    {
        Connection conn = Database.connectDatabase("DB_NAME0", "DB_USERNAME0", "DB_PASSWORD0", "DB_HOST0");
                
        try 
        {            
            Statement stm = conn.createStatement();
            
            String query = null;
            
            int conf_sec_proto_count = DatabaseQueries.countRows(DatabaseQueries.selectAllFromTable(conn, "config_security_protocol"));
            
            if (conf_sec_proto_count == 1)//if other parameters are inserted in 'config_security_protocol' pki_central_voters', etc.
                query = "UPDATE config_security_protocol SET pki_central_voters = "+pki_central_voters+", pki_locals_voters = "+pki_locals_voters+", vpn_central_locals = "+vpn_central_locals+"  WHERE config_sec_pr_id = 1";
            else if(conf_sec_proto_count == 0)
                query = "INSERT INTO config_security_protocol (pki_central_voters, pki_locals_voters, vpn_central_locals) VALUES ('" +pki_central_voters+ "', '" +pki_locals_voters+ "','" +vpn_central_locals+ "')";
            
            int ret = stm.executeUpdate(query);
            
            if(ret == 1)
                return true;
            else
                return false;
        }
        catch (SQLException ex)
        {
            System.out.println("Problem with a transaction");
            return false;
        }        
    }
    
    
          
}
