package org.pnyka.components.core.bulletinboard;

import java.io.IOException;
import java.math.BigInteger;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.text.Format;
import java.text.SimpleDateFormat;
import java.util.Date;
import javax.servlet.jsp.JspWriter;
import org.bouncycastle.crypto.params.ElGamalParameters;
import org.bouncycastle.crypto.params.ElGamalPublicKeyParameters;
import org.pnyka.components.core.votemanager.ProtocolParameters;
import org.pnyka.database.Database;
import org.pnyka.database.DatabaseQueries;

public class BulletinBoard 
{
    ProtocolParameters protoParams = null;
    Connection conn = null;

    /** Creates a new instance of BulletinBoard */
    public BulletinBoard()
    {
        try
        {
            ResultSet config_sec_protocol = null;
            boolean zkp1 = false, zkp2 = false, signatures = false, timestamp = false, reencryption = false;
            conn = Database.connectDatabase("DB_NAME0", "DB_USERNAME0", "DB_PASSWORD0", "DB_HOST0");
            config_sec_protocol = DatabaseQueries.selectAllFromTable(conn, "config_security_protocol");
            config_sec_protocol.next();
            zkp1 = config_sec_protocol.getBoolean("zkp_protocol");
            signatures = config_sec_protocol.getBoolean("signatures_protocol");
            timestamp = config_sec_protocol.getBoolean("timestamps_protocol");
            protoParams = new ProtocolParameters(reencryption, zkp1, zkp2, timestamp, signatures);
        }
        catch (SQLException ex){}
        
    }
    
     public void publishData(JspWriter out)
     { 
        try
        {
             ResultSet encrypted_votes = null, config_candidates = null; 
             
             config_candidates = DatabaseQueries.selectAllFromTable(conn, "config_candidates");
             int count_config_candidates = DatabaseQueries.countRows(config_candidates);
             
             encrypted_votes = PublishVote();
             
             int count = DatabaseQueries.countRows(encrypted_votes);
                          
             out.print("<table id='bulletin'>");               
             out.print("<tr>"); 
             out.print("<th><b>Voter ID</b></th>"); 
             out.print("<th><b>Encrypted Vote</b></th>"); 
              
             if(protoParams.timestamp == true){
                 out.print(" <th><b>Timestamp</b></th>"); }
             out.print("</tr>"); 

             for(int i=1; i<=count/count_config_candidates; i++ )
             {   
                 encrypted_votes.next();
                 out.print("<tr>"); 
                 out.print("<td align='center' width='50'>"); 
                 out.println(encrypted_votes.getString("ident_num")); 
                 out.print("</td>"); 
                 
                 out.print("<td align='center' width='300'><div class='scroll'>"); 
                 
                 for (int j=1;j<=count_config_candidates-1;j++)
                 {                 
                     out.println(encrypted_votes.getObject("int_z"));
                     out.println(encrypted_votes.getObject("int_c"));
                     encrypted_votes.next();
                 }
                 out.println(encrypted_votes.getObject("int_z"));
                 out.println(encrypted_votes.getObject("int_c"));   
                 
                 out.print("</div></td>"); 
                 
                 if(protoParams.timestamp == true)
                 {
                    out.print("<td align='center' width='200'>"); 
                    //out.println(encrypted_votes.getObject("timestamp"));
                    Date dt = new Date(encrypted_votes.getTimestamp("timestamp").getTime());
                    Format formatter = new SimpleDateFormat("dd-MM-yyyy, HH:mm:ss");
                    String s = formatter.format(dt);
                    out.print(s);
                    out.print("</td>"); 
                 }
                 out.print("</tr>"); 
             }  
           
               out.print(" </table>"); 
             
        }
        catch(SQLException ex1){}
        catch(IOException ex2){}
       
     }   
     
     //returns votes from database in order to be printed on bulletin board
     public ResultSet PublishVote()
     {
         ResultSet encrypted_votes = null;
         try
         {
            if(protoParams.timestamp == true)
                encrypted_votes = DatabaseQueries.selectFromTable(conn, "SELECT a.vote_id, a.voter_id, a.ident_num, a.int_z, a.int_c, a.timestamp FROM (SELECT ident_num,  max(timestamp) FROM encrypted_votes GROUP BY ident_num) AS b INNER JOIN encrypted_votes AS a ON b.ident_num = a.ident_num AND max = a.timestamp");
            else
                encrypted_votes = DatabaseQueries.selectAllFromTable(conn, "encrypted_votes");
         }
         catch (SQLException ex)
         {
            System.out.println("Problem with a transaction");
            ex.printStackTrace();
         }   
         
         return encrypted_votes;
     }

     public ElGamalPublicKeyParameters PublishPublicKey()
     {
         ResultSet public_key = null;
         BigInteger p = null, g = null, publicKey = null;
                  
         try
         {
            public_key = DatabaseQueries.selectAllFromTable(conn, "parameters");
            publicKey = public_key.getBigDecimal("public_key").toBigInteger();
            p = public_key.getBigDecimal("p").toBigInteger();
            g = public_key.getBigDecimal("g").toBigInteger();
         }
         catch (SQLException ex)
         {
            System.out.println("Problem with a transaction");
            ex.printStackTrace();
         }   
        
        return new ElGamalPublicKeyParameters(publicKey, new ElGamalParameters(p, g));
     }

     //returns a result set with candidates' description and the number of votes has took
     public ResultSet PublishResults()
     {
         ResultSet results = null;
         
         try
         {
            results = DatabaseQueries.selectFromTable(conn, "SELECT candidate_description, candidate_votes  FROM tallier, " +
                    "config_candidates WHERE tallier.candidate_id = config_candidates.candidate_id");
            
         }
         catch (SQLException ex)
         {
            System.out.println("Problem with a transaction");
            ex.printStackTrace();
         }   
         
         return results;
     }

}
