package org.pnyka.components.core.keyholder;

import java.math.BigDecimal;
import java.math.BigInteger;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import org.bouncycastle.crypto.params.ElGamalParameters;
import org.bouncycastle.crypto.params.ElGamalPrivateKeyParameters;
import org.bouncycastle.crypto.params.ElGamalPublicKeyParameters;
import org.pnyka.cryptolib.crypto.CipherText;
import org.pnyka.cryptolib.crypto.PlainText;
import org.pnyka.database.Database;
import org.pnyka.database.DatabaseQueries;

public class KeyHolder //extends BaseKeyHolder
{     
    private int id;
    private ElGamalPublicKeyParameters myPubKeyParams;
    private ElGamalPrivateKeyParameters myPrivKeyParams;
        
   /**
    * Constructs a new KeyHolder
    * @param id the unique id given to this BaseKeyHolder
    */
    public KeyHolder(int khId, ElGamalPublicKeyParameters myPubKeyParams, ElGamalPrivateKeyParameters myPrivKeyParams)
    {
        this.id = khId;
        this.myPubKeyParams = myPubKeyParams;
        this.myPrivKeyParams = myPrivKeyParams;
    }
    public ElGamalPublicKeyParameters computePartPublicKey(ElGamalPublicKeyParameters pubKeyParams)
    {        
        //ElGamalPublicKeyParameters myPubKeyParams = super.getMyPubKeyParams();
        BigInteger h = pubKeyParams.getY().multiply(myPubKeyParams.getY()).mod(myPubKeyParams.getParameters().getP());
             
        return new ElGamalPublicKeyParameters(h, myPubKeyParams.getParameters());
    }
    
    /**
    * Decrypts a message according to the ElGamal algorithm.
    * @param encMsg the encrypted message
    * @param params p, g values
    * @param privateKey the key for decryption
    * @return decrypted message
    */
   public PlainText Decrypt(CipherText encVote)
   {             
       return CipherText.elGamalDecrypt(encVote, myPrivKeyParams);
   }
   
   public CipherText DecryptPart(CipherText encVote)
   {             
       return CipherText.elGamalDecryptPart(encVote, myPrivKeyParams);
   }
    
   public ElGamalPublicKeyParameters  publickey()
   {             
       return myPubKeyParams;
   }
   public ElGamalPrivateKeyParameters  privatekey()
   {             
       return myPrivKeyParams;
   }
   
   //send decrypted results to tallier
   public static BigInteger[][] sendDecryptedResultToTallier()
   {
        Connection conn = Database.connectDatabase("DB_NAME0", "DB_USERNAME0", "DB_PASSWORD0", "DB_HOST0");
        ResultSet encrypted_result = null;
        BigInteger[][] decrypted = null; 
        
        try
        {
            int count_config_candidates = DatabaseQueries.countRows(DatabaseQueries.selectAllFromTable(conn, "config_candidates"));
            encrypted_result = DatabaseQueries.selectFromTable(conn, "SELECT * FROM tallier ORDER BY candidate_id");
            decrypted = new BigInteger[count_config_candidates][2];
            int ptr = 0, ind = 0;
            
            CipherText part;
            PlainText plain;  

            ResultSet kh = DatabaseQueries.selectAllFromTable(conn, "keyholders");
            KeyHolder[] newKHArr = new KeyHolder[DatabaseQueries.countRows(kh)];

            ResultSet params1 = DatabaseQueries.selectAllFromTable(conn, "parameters");
            params1.next();
            ElGamalParameters prms = new ElGamalParameters(((BigDecimal)params1.getObject("parameter_p")).toBigInteger(), ((BigDecimal)params1.getObject("parameter_g")).toBigInteger());

            ResultSet keyholders = DatabaseQueries.selectAllFromTable(conn, "keyholders");
            
            while(keyholders.next())
            {
                ResultSet keyholder = DatabaseQueries.selectFromTable(conn, "SELECT * FROM keyholders_keys WHERE keyholder_id ='"+keyholders.getInt("kh_id")+"' ");
                keyholder.next();
                ElGamalPublicKeyParameters myPubKeyParams = new ElGamalPublicKeyParameters (((BigDecimal)keyholder.getObject("public_key")).toBigInteger(), prms);
                ElGamalPrivateKeyParameters myPrivKeyParams = new ElGamalPrivateKeyParameters(((BigDecimal)keyholder.getObject("private_key")).toBigInteger(), prms);
                newKHArr[ind] = new KeyHolder(keyholders.getInt("kh_id"), myPubKeyParams, myPrivKeyParams);
                ind++;
            } 

            while(encrypted_result.next())
            { 
                 BigInteger z = encrypted_result.getBigDecimal("enc_res_z").toBigInteger();
                 BigInteger c = encrypted_result.getBigDecimal("enc_res_c").toBigInteger();

                 CipherText encResult = new CipherText(z, c);

                 part = encResult;

                 for(int i=0; i<newKHArr.length-1; i++)
                 {
                    part = newKHArr[i].DecryptPart(part);
                 }

                 plain = newKHArr[newKHArr.length-1].Decrypt(part);

                 int candidate_id = encrypted_result.getInt("candidate_id");
                 ResultSet config_candidates = DatabaseQueries.selectFromTable(conn, "SELECT * FROM config_candidates WHERE candidate_id ='"+candidate_id+"' ");
                 config_candidates.next();
                 int candidate_m = config_candidates.getInt("candidate_m");                 
              
                 decrypted[ptr][0] = BigInteger.valueOf(candidate_m);
                 decrypted[ptr][1] = plain.getM();
                 
                 ptr++;
                 /*BigInteger result = DiscreteLog.bruteForce(BigInteger.valueOf(candidate_m), plain.getM(), prms.getP());

                 int m = candidate_m;
                 int r = result.mod(prms.getP().subtract(BigInteger.ONE)).intValue();
                 tallier.publishResults(m, r);*/
              }
          return decrypted;  // (m, plaintext)     
       }
       catch(SQLException e)
       {     
           e.printStackTrace();
       }
       return decrypted;  // (m, plaintext)     
   }
   
}