package org.pnyka.components.core.votemanager;

import java.math.BigDecimal;
import java.math.BigInteger;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Timestamp;
import java.util.Date;
import org.bouncycastle.crypto.params.ElGamalParameters;
import org.bouncycastle.crypto.params.ElGamalPublicKeyParameters;
import org.pnyka.cryptolib.crypto.CipherText;
import org.pnyka.database.Database;
import org.pnyka.database.DatabaseQueries;

public class VoteManager
{
    ElGamalParameters prm = null;
    ElGamalPublicKeyParameters pubKeyParams = null;
    ProtocolParameters protoParams = null;
    Connection conn = null, connLocal = null;
    /** Creates a new instance of VoteManager */
    
    public VoteManager() 
    {
        try
        {
            boolean zkp1 = false, zkp2 = false, signatures = false, timestamp = false, reencryption = false;
            conn = Database.connectDatabase("DB_NAME0", "DB_USERNAME0", "DB_PASSWORD0", "DB_HOST0");
            ResultSet params = null, config_sec_protocol = null;
            params = DatabaseQueries.selectAllFromTable(conn, "parameters");
            params.next();
            prm = new ElGamalParameters(((BigDecimal)params.getObject("parameter_p")).toBigInteger(), ((BigDecimal)params.getObject("parameter_g")).toBigInteger());
            pubKeyParams = new ElGamalPublicKeyParameters(((BigDecimal)params.getObject("public_key")).toBigInteger(), prm);
               
            config_sec_protocol = DatabaseQueries.selectAllFromTable(conn, "config_security_protocol");
            config_sec_protocol.next();
            zkp1 = config_sec_protocol.getBoolean("zkp_protocol");
            signatures = config_sec_protocol.getBoolean("signatures_protocol");
            timestamp = config_sec_protocol.getBoolean("timestamps_protocol");
            protoParams = new ProtocolParameters(reencryption, zkp1, zkp2, timestamp, signatures);
            
            
            connLocal = Database.connectDatabase("DB_LOCAL_NAME", "DB_LOCAL_USERNAME", "DB_LOCAL_PASSWORD", "DB_LOCAL_HOST");
         } 
         catch (SQLException e){}
    }
    
    
public boolean PublishVote(int voter_id, String ident_num, CipherText[] vectorOfEncryptedVotes)
{
        boolean stored = true;
    
        if(protoParams.voteReEncryption == true)
            reEncryptVote(vectorOfEncryptedVotes);
        
        if(protoParams.timestamp == true)
            vectorOfEncryptedVotes = AppendTimestamp(vectorOfEncryptedVotes);
 
        String sqlText = null;
        
        try
        {
            connLocal.setAutoCommit(false);
            conn.setAutoCommit(false);
             
            Statement stm1 = connLocal.createStatement();
            Statement stm2 = conn.createStatement();
                    
                        
            for (int i = 0; i<vectorOfEncryptedVotes.length; i++)
            {
                CipherText encVote = vectorOfEncryptedVotes[i];
                                  
                if(encVote.getZ()!=null && encVote.getC()!=null)
                {
                    if(protoParams.timestamp == false)  
                        sqlText = "INSERT INTO encrypted_votes (voter_id, ident_num, int_z, int_c) VALUES ('" +voter_id + "','" +ident_num+ "','"+encVote.getZ()+"','" +encVote.getC()+ "')";
                    else            
                        sqlText = "INSERT INTO encrypted_votes (voter_id, ident_num, int_z, int_c, timestamp) VALUES ('" +voter_id + "','" +ident_num+ "','"+encVote.getZ()+"','" +encVote.getC()+ "', '"+encVote.getTimeStamp()+"')";
                      
                    stm1.addBatch(sqlText);
                    stm2.addBatch(sqlText);                     
                }
            } 
            
            stm1.execute("BEGIN;");
            int[] rows_affected1 = stm1.executeBatch();
            int[] rows_affected2 = null;
            
            ResultSet config_candidates = DatabaseQueries.selectAllFromTable(conn, "config_candidates");
            int count_config_candidates = DatabaseQueries.countRows(config_candidates);
            
            if(rows_affected1.length == count_config_candidates)               
            {                
                stm2.execute("BEGIN;");
                rows_affected2 = stm2.executeBatch();
                
                if(rows_affected2.length == count_config_candidates)
                {
                    connLocal.commit();                     
                    conn.commit();                    
                }
                else
                {
                    connLocal.rollback();
                    conn.rollback();
                    stored = false;
                }
            }               
            else
            {
                connLocal.rollback();                
                stored = false;
            }
           
        }
        catch (SQLException e)
        {
            stored = false;
        }
             
          return stored;
    }
    
    
    private CipherText[] AppendTimestamp(CipherText[] vectorOfEncryptedVotes)
    {
        Timestamp ts = new Timestamp(new Date().getTime());
        
        for(int i=0; i<vectorOfEncryptedVotes.length; i++)
            vectorOfEncryptedVotes[i] = new CipherText( vectorOfEncryptedVotes[i].getZ(),  vectorOfEncryptedVotes[i].getC(), ts);
       
        return vectorOfEncryptedVotes;//encrypted votes with timestamp
    }
    
    private CipherText[] reEncryptVote(CipherText[] vectorOfEncryptedVotes)
    {
        for(int i = 0; i<vectorOfEncryptedVotes.length; i++)
                vectorOfEncryptedVotes[i] = CipherText.elGamalEncrypt(vectorOfEncryptedVotes[i], pubKeyParams);
        
        return vectorOfEncryptedVotes;
    }
}
