package org.pnyka.components.logger;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import org.bouncycastle.crypto.params.ElGamalParameters;
import org.bouncycastle.crypto.params.ElGamalPublicKeyParameters;
import org.pnyka.components.core.votemanager.ProtocolParameters;
import org.pnyka.database.Database;
import org.pnyka.database.DatabaseQueries;

public class Logger 
{
    Connection conn = null;
    ElGamalParameters prm = null;
    ElGamalPublicKeyParameters pubKeyParams = null;
    ProtocolParameters protoParams = null;
    /** Creates a new instance of Logger */
    
    public Logger() 
    {
        conn = Database.connectDatabase("DB_NAME0", "DB_USERNAME0", "DB_PASSWORD0", "DB_HOST0");
        
        boolean zkp1 = false, zkp2 = false, signatures = false, timestamp = false, reencryption = false;
        ResultSet params = null, config_sec_protocol = null;

        try
        {
            config_sec_protocol = DatabaseQueries.selectAllFromTable(conn, "config_security_protocol");
            config_sec_protocol.next();
            zkp1 = config_sec_protocol.getBoolean("zkp_protocol");
            signatures = config_sec_protocol.getBoolean("signatures_protocol");
            timestamp = config_sec_protocol.getBoolean("timestamps_protocol");
            protoParams = new ProtocolParameters(reencryption, zkp1, zkp2, timestamp, signatures);
        }
        catch (SQLException ex)
        {
            System.out.println("Problem with a transaction");
            ex.printStackTrace();           
        }  
    }
    
   //string type can take 2 values: "EXT" for extented logging or "LIMIT" for limited      
   public void StartLoggingProcess(String type)
   {
       try
       {
           //retrieve encrypted votes from central ea
           ResultSet enc_votes_central = DatabaseQueries.selectAllFromTable(conn, "encrypted_votes");
           
           ResultSet local_eas = DatabaseQueries.selectFromTable(conn, "SELECT * FROM ea_definition WHERE is_local = TRUE");
           int count_local_eas = DatabaseQueries.countRows(local_eas);

           ResultSet[] local_selects = new ResultSet[count_local_eas]; //will contain batches of queries with inserts in election_catalog in each local ea's database 
           String vote_area;
           String db_name, db_username, db_psw, db_host;//for connections with local databases

           if(type.equals("EXT"))
           {               
                //first, create usernames and password for all voters
                ResultSet election_catalog = DatabaseQueries.selectAllFromTable(conn, "election_catalog");
                int count_voters = DatabaseQueries.countRows(election_catalog);

                while(local_eas.next())
                {   
                    vote_area = local_eas.getString("vote_area");                        

                    //Create connection with local ea's db
                    db_name = "DB_NAME"+vote_area.trim();
                    db_username = "DB_USERNAME"+vote_area.trim();
                    db_psw = "DB_PASSWORD"+vote_area.trim();
                    db_host = "DB_HOST"+vote_area.trim();
                    Connection local_db_conn = Database.connectDatabase(db_name, db_username, db_psw, db_host);

                    local_selects[local_eas.getRow()-1] = DatabaseQueries.selectAllFromTable(local_db_conn, "encrypted_votes");
                }
           }
           else if(type.equals("LIMIT"))
           {
               //first, create usernames and password for all voters
                ResultSet election_catalog = DatabaseQueries.selectAllFromTable(conn, "election_catalog");
                int count_voters = DatabaseQueries.countRows(election_catalog);

                while(local_eas.next())
                {   
                    vote_area = local_eas.getString("vote_area");                        

                    //Create connection with local ea's db
                    db_name = "DB_NAME"+vote_area.trim();
                    db_username = "DB_USERNAME"+vote_area.trim();
                    db_psw = "DB_PASSWORD"+vote_area.trim();
                    db_host = "DB_HOST"+vote_area.trim();
                    Connection local_db_conn = Database.connectDatabase(db_name, db_username, db_psw, db_host);

                    local_selects[local_eas.getRow()-1] = DatabaseQueries.selectAllFromTable(local_db_conn, "encrypted_votes");
                }
                
                for(int i=0; i<local_selects.length; i++)
                {
                    while(local_selects[i].next())
                        System.out.println("---"+local_selects[i].getInt("voter_id"));
                }
           }
       }
       catch (SQLException ex)
       {
            System.out.println("Problem with a transaction");
            ex.printStackTrace();           
       }  
   }
    
   
   public void VerifyTallier()
   {
       
   }
   
   public static void main (String[] args)
   {
       Logger log = new Logger();
       log.StartLoggingProcess("LIMIT");
   }
}
