package org.pnyka.components.core.votemanager;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.DataInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.math.BigDecimal;
import java.math.BigInteger;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Timestamp;
import java.util.Date;
import org.bouncycastle.crypto.params.ElGamalParameters;
import org.bouncycastle.crypto.params.ElGamalPrivateKeyParameters;
import org.bouncycastle.crypto.params.ElGamalPublicKeyParameters;
import org.pnyka.cryptolib.Signatures.ElGamalSigner;
import org.pnyka.cryptolib.Signatures.SignedMessage;
import org.pnyka.cryptolib.crypto.CipherText;
import org.pnyka.database.Database;
import org.pnyka.database.DatabaseQueries;

public class VoteManager
{
    ElGamalParameters prm = null;
    ElGamalPublicKeyParameters pubKeyParams = null;
    ProtocolParameters protoParams = null;
    Connection conn = null, connLocal = null;
    /** Creates a new instance of VoteManager */
    
    public VoteManager() 
    {
        try
        {
            boolean zkp1 = false, zkp2 = false, signatures = false, timestamp = false, reencryption = false;
            conn = Database.connectDatabase("DB_NAME0", "DB_USERNAME0", "DB_PASSWORD0", "DB_HOST0");
            ResultSet params = null, config_sec_protocol = null;
            params = DatabaseQueries.selectAllFromTable(conn, "parameters");
            params.next();
            prm = new ElGamalParameters(((BigDecimal)params.getObject("parameter_p")).toBigInteger(), ((BigDecimal)params.getObject("parameter_g")).toBigInteger());
            pubKeyParams = new ElGamalPublicKeyParameters(((BigDecimal)params.getObject("public_key")).toBigInteger(), prm);
               
            config_sec_protocol = DatabaseQueries.selectAllFromTable(conn, "config_security_protocol");
            config_sec_protocol.next();
            zkp1 = config_sec_protocol.getBoolean("zkp_protocol");
            signatures = config_sec_protocol.getBoolean("signatures_protocol");
            timestamp = config_sec_protocol.getBoolean("timestamps_protocol");
            protoParams = new ProtocolParameters(reencryption, zkp1, zkp2, timestamp, signatures);
            
            
            connLocal = Database.connectDatabase("DB_LOCAL_NAME", "DB_LOCAL_USERNAME", "DB_LOCAL_PASSWORD", "DB_LOCAL_HOST");
         } 
         catch (SQLException e){}
    }
    
    //store votes in database 
    public boolean PublishVote(int voter_id, String ident_num, CipherText[] vectorOfEncryptedVotes)
    {
        boolean stored = true;
    
        if(protoParams.voteReEncryption == true)
            ReEncryptVote(vectorOfEncryptedVotes);
        
        if(protoParams.timestamp == true)
            vectorOfEncryptedVotes = AppendTimestamp(vectorOfEncryptedVotes);
 
        String sqlText = null;
        
        try
        {
            connLocal.setAutoCommit(false);
            conn.setAutoCommit(false);
             
            Statement stm1 = connLocal.createStatement();
            Statement stm2 = conn.createStatement();
                    
                        
            for (int i = 0; i <  vectorOfEncryptedVotes.length; i++)
            {
                CipherText encVote = vectorOfEncryptedVotes[i];
                                  
                if(encVote.getZ()!=null && encVote.getC()!=null)
                {
                    if(protoParams.timestamp == false)  
                        sqlText = "INSERT INTO encrypted_votes (voter_id, ident_num, int_z, int_c) VALUES ('" +voter_id + "','" +ident_num+ "','"+encVote.getZ()+"','" +encVote.getC()+ "')";
                    else            
                        sqlText = "INSERT INTO encrypted_votes (voter_id, ident_num, int_z, int_c, timestamp) VALUES ('" +voter_id + "','" +ident_num+ "','"+encVote.getZ()+"','" +encVote.getC()+ "', '"+encVote.getTimeStamp()+"')";
                      
                    stm1.addBatch(sqlText);
                    stm2.addBatch(sqlText);                     
                }
            } 
            
            stm1.execute("BEGIN;");
            int[] rows_affected1 = stm1.executeBatch();
            int[] rows_affected2 = null;
            
            ResultSet config_candidates = DatabaseQueries.selectAllFromTable(conn, "config_candidates");
            int count_config_candidates = DatabaseQueries.countRows(config_candidates);
            
            if(rows_affected1.length == count_config_candidates)               
            {                
                stm2.execute("BEGIN;");
                rows_affected2 = stm2.executeBatch();
                
                if(rows_affected2.length == count_config_candidates)
                {
                    connLocal.commit();                     
                    conn.commit();                    
                }
                else
                {
                    connLocal.rollback();
                    conn.rollback();
                    stored = false;
                }
            }
               
            else
            {
                connLocal.rollback();                
                stored = false;
            }
           
        }
        catch (SQLException e)
        {
            stored = false;
        }

        return stored;
    }
    
    //appends sql timestamp
    private CipherText[] AppendTimestamp(CipherText[] vectorOfEncryptedVotes)
    {
        Timestamp ts = new Timestamp(new Date().getTime());
        
        for(int i=0; i<vectorOfEncryptedVotes.length; i++)        
            vectorOfEncryptedVotes[i] = new CipherText( vectorOfEncryptedVotes[i].getZ(),  vectorOfEncryptedVotes[i].getC(), ts);
        
        return vectorOfEncryptedVotes;//encrypted votes with timestamp
    }
    
    //reencrypts vector of votes
    private CipherText[] ReEncryptVote(CipherText[] vectorOfEncryptedVotes)
    {
        for(int i = 0; i<vectorOfEncryptedVotes.length; i++)
                vectorOfEncryptedVotes[i] = CipherText.elGamalEncrypt(vectorOfEncryptedVotes[i], pubKeyParams);
        
        return vectorOfEncryptedVotes;
    }
    
    //signs vector of votes
    private SignedMessage[] SignVote(CipherText[] vectorOfEncryptedVotes) throws SQLException
    {   
        //initializations
        SignedMessage[] sm = new SignedMessage[vectorOfEncryptedVotes.length];
        BigInteger[] sign = null;
        
        //retrieve private key
        ResultSet keyholders_keys = DatabaseQueries.selectAllFromTable(conn, "keyholders_keys");
        ElGamalParameters params = pubKeyParams.getParameters();
        BigInteger x = BigInteger.ZERO;
        
        while(keyholders_keys.next())
            x = x.add(keyholders_keys.getBigDecimal("private_key").toBigInteger());
       
        ElGamalPrivateKeyParameters privKeyParams = new ElGamalPrivateKeyParameters(x, params);
        
        //initialize signer
        ElGamalSigner signer = new ElGamalSigner();
        signer.init(true, privKeyParams);
        
        //sign
        for(int i = 0; i<vectorOfEncryptedVotes.length; i++)
        {
            sign = signer.generateSignature(vectorOfEncryptedVotes[i]);
            sm[i] = new SignedMessage(vectorOfEncryptedVotes[i], sign);
        }
        
        /*signer.init(false, pubKeyParams);
        for(int i = 0; i<sm.length; i++)
        {
            boolean bl = signer.verifySignature(vectorOfEncryptedVotes[i], sm[i].getSign());
            System.out.println(bl);
        }*/
        
        return sm;
    }
    
    //creates receipt for voter with id "voter_id"
    public void CreateReceipt(int voter_id) throws SQLException, IOException
    {
        File file = new File("user"+voter_id+".receipt"); 
        BufferedWriter out = new BufferedWriter(new FileWriter("user"+voter_id+".receipt"));
                 
        //find how many columns has the "encrypted_votes" table
        ResultSet receipt_data = DatabaseQueries.selectFromTable(conn, "SELECT * FROM encrypted_votes WHERE voter_id = '"+voter_id+"'");
        ResultSetMetaData rsmd = receipt_data.getMetaData();
        int num_of_columns = rsmd.getColumnCount(); 
        
        while(receipt_data.next())
        {
            for(int k=1; k<=num_of_columns; k++)
            {
                if(k == 1)
                    out.write(receipt_data.getInt(k)+",");
                else if(k == 2)
                    out.write(receipt_data.getInt(k)+",");
                else if(k == 3)
                    out.write(receipt_data.getString(k).trim()+",");
                else if(k == 4)
                    out.write(receipt_data.getBigDecimal(k)+",");
                else if(k == 5)
                    out.write(receipt_data.getBigDecimal(k)+",");
                else if(k == 6)
                    out.write(receipt_data.getTimestamp(k)+",");
                else if(k == 7)
                    out.write(receipt_data.getBoolean(k)+",");
                else if(k == 8)
                    out.write(receipt_data.getBoolean(k)+",");
                else if(k == 9)
                    out.write(receipt_data.getBigDecimal(k)+",");
                else if(k == 10)
                    out.write(receipt_data.getBigDecimal(k)+"");                                
            }
            out.newLine();
        }               
        out.close(); 
    }
    
    public boolean CheckReceipt(int voter_id) 
    {
         try
         {
            // Open the file 
            // command line parameter
            FileInputStream fstream = new FileInputStream("user"+voter_id+".receipt");
            // Get the object of DataInputStream
            DataInputStream in = new DataInputStream(fstream);
            BufferedReader br = new BufferedReader(new InputStreamReader(in));
            String strLine;
            BigInteger[] sign = new BigInteger[2];
            //Read File Line By Line
           /* while ((strLine = br.readLine()) != null)  
            {
                String[] str_array = strLine.split(",");
                //get r, s
                if(str_array[8] != null && str_array[9] != null)
                {
                    sign[0] = new BigInteger(str_array[8]); 
                    sign[1] = new BigInteger(str_array[9]);
                }
                
            }*/

            in.close();
         }
         catch (Exception e)
         {
              e.printStackTrace();
         }
         
         return true;
    }
    
}
