package org.pnyka.components.logger;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.util.Properties;
import org.bouncycastle.crypto.params.ElGamalParameters;
import org.bouncycastle.crypto.params.ElGamalPublicKeyParameters;
import org.pnyka.components.administrator.BasicAdministrationOperations;
import org.pnyka.components.core.votemanager.ProtocolParameters;
import org.pnyka.database.Database;
import org.pnyka.database.DatabaseQueries;
import org.pnyka.util.PropertiesFiles;

public class Logger 
{
    Connection conn = null;
    ElGamalParameters prm = null;
    ElGamalPublicKeyParameters pubKeyParams = null;
    ProtocolParameters protoParams = null;
    /** Creates a new instance of Logger */
    
    public Logger() 
    {
        conn = Database.connectDatabase("DB_NAME0", "DB_USERNAME0", "DB_PASSWORD0", "DB_HOST0");
        
        boolean zkp1 = false, zkp2 = false, signatures = false, timestamp = false, reencryption = false;
        ResultSet params = null, config_sec_protocol = null;

        try
        {
            config_sec_protocol = DatabaseQueries.selectAllFromTable(conn, "config_security_protocol");
            config_sec_protocol.next();
            zkp1 = config_sec_protocol.getBoolean("zkp_protocol");
            signatures = config_sec_protocol.getBoolean("signatures_protocol");
            timestamp = config_sec_protocol.getBoolean("timestamps_protocol");
            protoParams = new ProtocolParameters(reencryption, zkp1, zkp2, timestamp, signatures);            
        }
        catch (SQLException ex)
        {
            System.out.println("Problem with a transaction");
            ex.printStackTrace();           
        }  
    }
    
   //string type can take 2 values: "EXT" for extented logging or "LIMIT" for limited      
   public void StartLoggingProcess(String type)
   {
       try
       {           
           ResultSet local_selects = null, enc_votes = null; 
           String db_name, db_username, db_psw, db_host;//for connections with local databases
           PropertiesFiles pf = new PropertiesFiles();
           Properties properties = pf.ReadProperties("system.properties");
           String db_num = properties.getProperty("DB_NUM");//in other words number of localea's                       
           String logfile_path = "";
           
           if(type.equals("EXT"))
           {      
               //find how many columns has the "encrypted_votes" table
               enc_votes = DatabaseQueries.selectAllFromTable(conn, "encrypted_votes");
               ResultSetMetaData rsmd = enc_votes.getMetaData();
               int num_of_columns = rsmd.getColumnCount();               
                             
               for(int i=0; i<Integer.parseInt(db_num); i++)
               {  
                    //Create connection with local ea's db
                    db_name = "DB_NAME"+i;
                    db_username = "DB_USERNAME"+i;
                    db_psw = "DB_PASSWORD"+i;
                    db_host = "DB_HOST"+i;
                    Connection local_db_conn = Database.connectDatabase(db_name, db_username, db_psw, db_host);

                    local_selects = DatabaseQueries.selectAllFromTable(local_db_conn, "encrypted_votes");
                    
                    logfile_path = properties.getProperty("LOG_"+i);
                               
                    File file = new File(logfile_path); 
                    BufferedWriter out = new BufferedWriter(new FileWriter(logfile_path));
                 
                    while(local_selects.next())
                    {
                        for(int k=1; k<=num_of_columns; k++)
                        {
                            //out.write(local_selects.getInt("vote_id")+","+local_selects.getInt("voter_id")+","+local_selects.getBigDecimal("int_z").toBigInteger()+","+local_selects.getBigDecimal("int_c").toBigInteger());
                            if(k == 1)
                                out.write(local_selects.getInt(k)+",");
                            else if(k == 2)
                                out.write(local_selects.getInt(k)+",");
                            else if(k == 3)
                                out.write(local_selects.getString(k).trim()+",");
                            else if(k == 4)
                                out.write(local_selects.getBigDecimal(k)+",");
                            else if(k == 5)
                                out.write(local_selects.getBigDecimal(k)+",");
                            else if(k == 6)
                                out.write(local_selects.getTimestamp(k)+",");
                            else if(k == 7)
                                out.write(local_selects.getBoolean(k)+",");
                            else if(k == 8)
                                out.write(local_selects.getBoolean(k)+",");
                            else if(k == 9)
                                out.write(local_selects.getBigDecimal(k)+",");
                            else if(k == 10)
                                out.write(local_selects.getBigDecimal(k)+"");                                
                        }
                        
                        out.newLine();
                    }               
                    out.close();                      
                }  
           }
           else if(type.equals("LIMIT"))//logs either vote or vote with timestamp
           { 
                for(int i=0; i<Integer.parseInt(db_num); i++)
                {                      
                    //Create connection with local ea's db
                    db_name = "DB_NAME"+i;
                    db_username = "DB_USERNAME"+i;
                    db_psw = "DB_PASSWORD"+i;
                    db_host = "DB_HOST"+i;
                    Connection local_db_conn = Database.connectDatabase(db_name, db_username, db_psw, db_host);

                    local_selects = DatabaseQueries.selectAllFromTable(local_db_conn, "encrypted_votes");
                    
                    logfile_path = properties.getProperty("LOG_"+i);
                               
                    File file = new File(logfile_path); 
                    BufferedWriter out = new BufferedWriter(new FileWriter(logfile_path));
                 
                    while(local_selects.next())
                    {
                        if(protoParams.timestamp == true)
                            out.write(local_selects.getInt("vote_id")+","+local_selects.getInt("voter_id")+","+local_selects.getBigDecimal("int_z").toBigInteger()+","+local_selects.getBigDecimal("int_c").toBigInteger()+","+local_selects.getTimestamp("timestamp"));
                        else
                            out.write(local_selects.getInt("vote_id")+","+local_selects.getInt("voter_id")+","+local_selects.getBigDecimal("int_z").toBigInteger()+","+local_selects.getBigDecimal("int_c").toBigInteger());
                        
                        out.newLine();
                    }               
                    out.close();                      
                }                        
           }
       }
       catch (SQLException ex)
       {
            ex.printStackTrace();           
       }  
       catch (IOException e) 
       { 
            e.printStackTrace();
       }
   }
    
   
   public void VerifyTallier()
   {
       BasicAdministrationOperations.startTallier();
   }   
   
}
